/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.junit.rules.ftpserver;

import java.io.File;
import java.io.IOException;

import org.apache.ftpserver.FtpServer;
import org.apache.ftpserver.FtpServerFactory;
import org.apache.ftpserver.filesystem.nativefs.NativeFileSystemFactory;
import org.apache.ftpserver.ftplet.FtpException;
import org.apache.ftpserver.ftplet.User;
import org.apache.ftpserver.ftplet.UserManager;
import org.apache.ftpserver.listener.ListenerFactory;
import org.junit.Rule;
import org.junit.rules.ExternalResource;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.junit.rules.ftpserver.usermanager.BasicUserManagerFactory;

/**
 * The {@link FTPServer} {@link Rule} allows creation of a FTP server resource. It is guaranteed to be deleted when the
 * test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public class MyTestSuite {
 *  &#064;Rule
 *  public FTPServer ftpServer = new FTPServer(...);
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      ftpServer.registerUser(userName, userPassword);
 *      ftpServer.putFile(path, file);
 *      ftpServer.getFile(path, file);
 *      ...
 *  }
 * }
 * </pre>
 */
public class FTPServer extends ExternalResource {

    /**
     * The default FTP port
     */
    public static final int DEFAULT_FTP_PORT = 10180;

    /**
     * The name of the 'admin' user.
     */
    public static final String ADMIN_NAME = "admin";

    /**
     * The port on which FTP requests are listen
     */
    private final int ftpPort;

    /**
     * The FTP server
     */
    private final FtpServer server;

    /**
     * The user manager
     */
    private final UserManager userManager;

    /**
     * A temporary folder that will be used to define the root of the file-system used by the FTP server.
     */
    private final TemporaryFolder temporaryFolder = new TemporaryFolder();

    /**
     * The root directory of the file-system used by the FTP server.
     */
    private File rootFileSystem = null;

    /**
     * Creates a temporary ftp-server listening FTP requests on the given port.
     * 
     * @param ftpPort
     *            The HTTP port on which requests will be listen
     */
    public FTPServer(final int ftpPort) {
        this.ftpPort = ftpPort;

        final FtpServerFactory serverFactory = new FtpServerFactory();
        final ListenerFactory listenerFactory = new ListenerFactory();
        // set the port of the listener
        listenerFactory.setPort(this.ftpPort);
        // replace the default listener
        serverFactory.addListener("default", listenerFactory.createListener());
        // define the user manager
        this.userManager = new BasicUserManagerFactory(ADMIN_NAME).createUserManager();
        serverFactory.setUserManager(this.userManager);
        // file system
        final NativeFileSystemFactory nfsf = new NativeFileSystemFactory();
        nfsf.setCreateHome(true);
        serverFactory.setFileSystem(nfsf);
        // start the server
        this.server = serverFactory.createServer();
    }

    /**
     * Creates a temporary web-server listening HTTP requests on the default port ({@value #DEFAULT_HTTP_PORT})
     */
    public FTPServer() {
        this(DEFAULT_FTP_PORT);
    }

    @Override
    protected void before() throws Throwable {
        this.temporaryFolder.create();
        this.rootFileSystem = this.temporaryFolder.newFolder();
        this.initializeFtpServer();
    }

    @Override
    protected void after() {
        try {
            this.destroyFtpServer();
        } catch (final Exception e) {
            e.printStackTrace();
        }
        this.temporaryFolder.delete();
    }

    /**
     * Initialize & start the FTP server
     */
    private void initializeFtpServer() throws Exception {

        this.server.start();
    }

    /**
     * Free the web server
     */
    private void destroyFtpServer() throws Exception {

        this.server.stop();
    }

    /**
     * @return The FTP port on which the FTP server is listening
     */
    public int getFtpPort() {
        return this.ftpPort;
    }

    /**
     * <p>
     * Register a new user that will accepted on new connections.
     * </p>
     * <p>
     * If a user home directory is set, it will be created if it does not exist.
     * </p>
     * 
     * @param user
     *            The new user with its all data
     * @throws FtpException
     */
    public void registerUser(final User user) throws FtpException {
        this.userManager.save(user);

        // If the user home directory does not exist, we create it
        final File userHomeDirectory = new File(user.getHomeDirectory());
        if (!userHomeDirectory.exists()) {
            userHomeDirectory.mkdirs();
        }
    }

    /**
     * Unregister a user that will be rejected on new connections
     * 
     * @param userName
     *            The user name of the user to unregister
     * @throws FtpException
     */
    public void unregisterUser(final String userName) throws FtpException {
        this.userManager.delete(userName);
    }

    public File getRootFileSystem() {
        return this.rootFileSystem;
    }

    /**
     * Create and register a temporary file in the home directory of the given user. The created file is empty.
     * 
     * @param user
     *            The owner of the file
     * @return The name of temporary file created, relative to the user home directory.
     * @throws IOException
     *             The temporary file can not be created
     */
    public String registerTempFile(final User user) throws IOException {
        final File userHomeDirectory = new File(user.getHomeDirectory());
        final File tempFile = File.createTempFile(user.getName(), ".tmp", userHomeDirectory);

        return userHomeDirectory.toURI().relativize(tempFile.toURI()).toString();
    }

    /**
     * Create and register a temporary file in the home directory of the given user. The created file is empty. The
     * created file will have the given extension.
     * 
     * @param user
     *            The owner of the file
     * @param extension
     *            The extension of the file to create. The '.' delimiting the base name and the extension will be
     *            automatically added.
     * @return The name of temporary file created, relative to the user home directory.
     * @throws IOException
     *             The temporary file can not be created
     */
    public String registerTempFile(final User user, final String extension) throws IOException {
        final File userHomeDirectory = new File(user.getHomeDirectory());
        final File tempFile = File.createTempFile(user.getName(), "." + extension, userHomeDirectory);

        return userHomeDirectory.toURI().relativize(tempFile.toURI()).toString();
    }

    /**
     * Create and register a file with the given name in the home directory of the given user. The created file is
     * empty.
     * 
     * @param user
     *            The owner of the file
     * @param filename
     *            The file name.
     * @return <code>true</code> if the file was successfully created, <code>false</code> otherwise.
     * @throws IOException
     *             An error occurs creating the file
     */
    public boolean registerFile(final User user, final String filename) throws IOException {

        final File userHomeDirectory = new File(user.getHomeDirectory());
        return new File(userHomeDirectory, filename).createNewFile();
    }

}
