/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.junit.rules.log.handler;

import java.util.List;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

import org.junit.Rule;
import org.junit.rules.ExternalResource;

/**
 * The {@link InMemoryLogHandler} {@link Rule} allows creation of an in-memory log handler. It is guaranteed to be
 * deleted when the test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public class MyTestSuite {
 *  &#064;Rule
 *  public InMemoryLogHandler inMemoryLogHandler = new InMemoryLogHandler();
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      final Logger logger = Logger.getAnonymousLogger();
 *      logger.addHandler(this.inMemoryLogHandler.getHandler());
 *      logger.setLevel(Level.MONIT);
 *      ...
 *      final List<LogRecord> allLogRecords = this.inMemoryLogHandler.getAllRecords(Level.MONIT);
 *      assertEquals(2, allLogRecords.size());
 *  }
 * }
 * </pre>
 */
public class InMemoryLogHandler extends ExternalResource {

    /**
     * The real in-memory log handler
     */
    private final InMemoryRealLogHandler inMemoryRealLogHandler = new InMemoryRealLogHandler();

    @Override
    protected void before() throws Throwable {
        // NOP
    }

    @Override
    protected void after() {
        this.inMemoryRealLogHandler.clearRecords();
    }

    /**
     * @return The real log {@link Handler} used for logging.
     */
    public Handler getHandler() {
        return this.inMemoryRealLogHandler;
    }

    /**
     * @return All logged records.
     */
    public List<LogRecord> getAllRecords() {
        return this.inMemoryRealLogHandler.getAllRecords();
    }

    /**
     * All logged records for a specific given {@link Level}.
     * 
     * @param level
     *            The {@link Level} of seeked records.
     * @return All logged records for the specific given
     */
    public List<LogRecord> getAllRecords(final Level level) {
        return this.inMemoryRealLogHandler.getAllRecords(level);
    }

    /**
     * Clear all log records
     */
    public void clear() {
        this.inMemoryRealLogHandler.clearRecords();
    }

}
