/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.junit.rules.log.handler;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

/**
 * An in-memory log handler that provide several methods to interact with a ordered {@link List} which store
 * {@link LogRecord} in memory.
 * 
 * @author Adrien Ruffie - EBM WebSourcing
 * @author Christophe DENEUX - Linagora
 */
public final class InMemoryRealLogHandler extends Handler {

    // The log records stored
    private final List<LogRecord> records;

    public InMemoryRealLogHandler() {
        this.records = new CopyOnWriteArrayList<LogRecord>();
    }

    /**
     * @return All logged records.
     */
    public List<LogRecord> getAllRecords() {
        return Collections.unmodifiableList(this.records);
    }

    /**
     * All logged records for a specific given {@link Level}.
     * 
     * @param level
     *            The {@link Level} of seeked records.
     * @return All logged records for the specific given
     */
    public List<LogRecord> getAllRecords(final Level level) {

        final List<LogRecord> resultRecords = new ArrayList<LogRecord>();

        final Iterator<LogRecord> itRecords = this.records.iterator();
        while (itRecords.hasNext()) {
            final LogRecord logRecord = itRecords.next();
            if (logRecord.getLevel().equals(level)) {
                resultRecords.add(logRecord);
            }
        }

        return resultRecords;
    }

    /**
     * Allow to clear the buffer.
     */
    public void clearRecords() {
        this.records.clear();
    }

    @Override
    public void publish(final LogRecord record) {
        if (isLoggable(record)) {

            /*
             * Put the new entry in <List> in order to keep this entry in memory
             */
            this.records.add(record);
        }
    }

    @Override
    public void flush() {
        // Nothing to do for this implementation
    }

    @Override
    public void close() throws SecurityException {
        // Nothing to do for this implementation
    }
}
