/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.junit.rules.log.handler;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.junit.Rule;
import org.junit.Test;

/**
 * Unit tests of {@link InMemoryLogHandler}.
 * 
 * @author Adrien Ruffie - EBM WebSourcing
 * @author Christophe DENEUX - Linagora
 * 
 */
public class InMemoryLogHandlerTest {

    @Rule
    final public InMemoryLogHandler inMemoryLogHandler = new InMemoryLogHandler();

    // Published message prefix constant
    private final String LOG_RECORD_MSG = "TestPublishLogRecord record: ";

    /**
     * Check that {@link InMemoryLogHandler} is empty when starting a test.
     */
    @Test
    public void startingTest() {
        assertEquals("InMemoryLogHandler is empty after initialization", 0, this.inMemoryLogHandler.getAllRecords()
                .size());
    }

    @Test
    public void testGetAllRecords() {
        final Logger logger = Logger.getAnonymousLogger();
        logger.addHandler(this.inMemoryLogHandler.getHandler());

        assertEquals("InMemoryLogHandler isn't empty after initialization", 0, this.inMemoryLogHandler.getAllRecords()
                .size());

        logger.info(LOG_RECORD_MSG + "1");
        assertEquals("InMemoryLogHandler hasn't 1 log record", 1, this.inMemoryLogHandler.getAllRecords().size());

        logger.info(LOG_RECORD_MSG + "2");
        assertEquals("InMemoryLogHandler hasn't 2 log records", 2, this.inMemoryLogHandler.getAllRecords().size());

        logger.info(LOG_RECORD_MSG + "3");
        assertEquals("InMemoryLogHandler hasn't 3 log records", 3, this.inMemoryLogHandler.getAllRecords().size());

        logger.info(LOG_RECORD_MSG + "4");
        assertEquals("InMemoryLogHandler hasn't 4 log records", 4, this.inMemoryLogHandler.getAllRecords().size());

        logger.info(LOG_RECORD_MSG + "5");
        assertEquals("InMemoryLogHandler hasn't 5 log records", 5, this.inMemoryLogHandler.getAllRecords().size());
    }

    @Test
    public void testGetAllRecordsForLevel() {
        final Logger logger = Logger.getAnonymousLogger();
        logger.addHandler(this.inMemoryLogHandler.getHandler());

        assertEquals("InMemoryLogHandler is empty after initialization", 0, this.inMemoryLogHandler.getAllRecords()
                .size());

        logger.info(LOG_RECORD_MSG + "1");
        logger.info(LOG_RECORD_MSG + "2");
        logger.info(LOG_RECORD_MSG + "3");
        logger.finest(LOG_RECORD_MSG + "4");
        logger.finest(LOG_RECORD_MSG + "5");
        logger.severe(LOG_RECORD_MSG + "6");

        assertEquals("3 records aren't recovered for INFO log level", 3,
                this.inMemoryLogHandler.getAllRecords(Level.INFO).size());
        assertEquals("2 records aren't recovered for FINEST log level", 0,
                this.inMemoryLogHandler.getAllRecords(Level.FINEST).size());
        assertEquals("1 records aren't recovered for SEVERE log level", 1,
                this.inMemoryLogHandler.getAllRecords(Level.SEVERE).size());

        logger.setLevel(Level.FINEST);
        logger.finest(LOG_RECORD_MSG + "7");
        logger.finest(LOG_RECORD_MSG + "8");
        assertEquals("2 records aren't recovered for FINEST log level", 2,
                this.inMemoryLogHandler.getAllRecords(Level.FINEST).size());

        this.inMemoryLogHandler.clear();
        this.inMemoryLogHandler.getHandler().setLevel(Level.INFO);
        logger.finest(LOG_RECORD_MSG + "7");
        logger.finest(LOG_RECORD_MSG + "8");
        assertEquals("2 records aren't recovered for FINEST log level", 0,
                this.inMemoryLogHandler.getAllRecords(Level.FINEST).size());
    }

    @Test
    public void testClearRecords() {
        final Logger logger = Logger.getAnonymousLogger();
        logger.addHandler(this.inMemoryLogHandler.getHandler());

        assertEquals("InMemoryLogHandler isn't empty after initialization", 0, this.inMemoryLogHandler.getAllRecords()
                .size());

        logger.info(LOG_RECORD_MSG + "1");
        assertEquals("InMemoryLogHandler hasn't one log record", 1, this.inMemoryLogHandler.getAllRecords().size());

        logger.info(LOG_RECORD_MSG + "2");
        assertEquals("InMemoryLogHandler hasn't two log record", 2, this.inMemoryLogHandler.getAllRecords().size());

        this.inMemoryLogHandler.clear();
        assertEquals("InMemoryLogHandler isn't empty after cleaning", 0, this.inMemoryLogHandler.getAllRecords().size());
    }

    @Test
    public void testPublishLogRecord() {
        final Logger logger = Logger.getAnonymousLogger();
        logger.addHandler(this.inMemoryLogHandler.getHandler());

        assertEquals("InMemoryLogHandler isn't empty after initialization", 0, this.inMemoryLogHandler.getAllRecords()
                .size());

        logger.info(LOG_RECORD_MSG + "1");
        assertEquals("InMemoryLogHandler hasn't one log record", 1, this.inMemoryLogHandler.getAllRecords().size());
        assertEquals(LOG_RECORD_MSG + " uncorrectly published", LOG_RECORD_MSG + "1", this.inMemoryLogHandler
                .getAllRecords().get(0).getMessage());
    }

    @Test
    public void testPublishLogRecordLevelOFF() {
        final Logger logger = Logger.getAnonymousLogger();
        logger.addHandler(this.inMemoryLogHandler.getHandler());
        logger.setLevel(Level.OFF);

        logger.info(LOG_RECORD_MSG + "1");
        assertTrue(this.inMemoryLogHandler.getAllRecords().isEmpty());
    }

}
